/*++========================================================================
 * Program Name:     Novell NCP Redirector for Linux
 * File Name:        proc.c
 * Version:          v1.00
 * Author:           James Turner
 *
 * Abstract:         This module contains functions that create the
 *                   interface to the proc filesystem.
 * Notes:
 * Revision History: 
 *
 *
 * Copyright (C) 2005 Novell, Inc.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *======================================================================--*/

/*===[ Include files specific to Linux ]==================================*/
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/proc_fs.h>
#include <linux/smp_lock.h>

/*===[ Include files specific to this module ]============================*/
#include "vfs.h"

/*===[ External data ]====================================================*/
extern char *Novfs_CurrentMount;

/*===[ External prototypes ]==============================================*/
extern int DbgPrint( char *Fmt, ... );

extern ssize_t
Daemon_Receive_Reply(struct file *file, const char *buf, size_t nbytes, loff_t *ppos);

extern ssize_t 
Daemon_Send_Command(struct file *file, char *buf, size_t len, loff_t *off);

extern int Daemon_ioctl(struct inode *inode, struct file *file, unsigned int cmd, unsigned long arg);

extern int     Daemon_Library_close(struct inode *inode, struct file *file);
extern int     Daemon_Library_ioctl(struct inode *inode, struct file *file, unsigned int cmd, unsigned long arg);
extern int     Daemon_Library_open(struct inode *inode, struct file *file);
extern ssize_t Daemon_Library_read(struct file *file, char *buf, size_t len, loff_t *off);
extern ssize_t Daemon_Library_write(struct file *file, const char *buf, size_t len, loff_t *off);

extern int Daemon_Open_Control(struct inode *Inode, struct file *File);
extern int Daemon_Close_Control(struct inode *Inode, struct file *File);

extern int Daemon_getversion( char *Buf, int Length );

/*===[ Manifest constants ]===============================================*/

/*===[ Type definitions ]=================================================*/

/*===[ Function prototypes ]==============================================*/
ssize_t
Novfs_User_proc_read(struct file * file, char * buf, size_t nbytes, loff_t *ppos);

ssize_t
Novfs_User_proc_write(struct file * file, char * buf, size_t nbytes, loff_t *ppos);

int Novfs_User_proc_ioctl(struct inode *inode, struct file *file, unsigned int cmd, unsigned long arg);

int Init_Procfs_Interface( void );
void Uninit_Procfs_Interface( void );

/*===[ Global variables ]=================================================*/
struct proc_dir_entry *Novfs_Procfs_dir, *Novfs_Control, *Novfs_Library, *Novfs_Version;
static struct file_operations Daemon_proc_fops;
static struct file_operations Library_proc_fops;


/*===[ Code ]=============================================================*/

/*++======================================================================*/
int Novfs_Get_Version(char *page, char **start, off_t off, int count, int *eof, void *data)
/*
 * Arguments:
 * 
 * Returns:
 * 
 * Abstract:
 * 
 * Notes:
 * 
 * Environment:
 * 
 *========================================================================*/
{
   char *buf, tbuf[48];
   int len=0, i;

   if ( !off )   
   {
      buf = page+off;
      *start = buf;
      len = sprintf(buf, "Novfs Version=%s\n", NOVFS_VERSION_STRING);
      i = Daemon_getversion(tbuf, sizeof(tbuf));
      if ((i > 0) && i < (count-len))
      {
         len += sprintf(buf+len, "Novfsd Version=%s\n", tbuf);
      }
      
      if (Novfs_CurrentMount)
      {
         i = strlen(Novfs_CurrentMount);
         if ((i > 0) && i < (count-len))
         {
            len += sprintf(buf+len, "Novfs mount=%s\n", Novfs_CurrentMount);
         }
      }
      DbgPrint("Novfs_Get_Version:\n%s\n", buf);
   }
	*eof = 1;
   return(len);
}

/*++======================================================================*/
ssize_t
Novfs_User_proc_read(struct file * file, char * buf, size_t nbytes, loff_t *ppos)
/*
 *
 *  Arguments:
 *
 *  Returns:
 *
 *  Abstract:
 *
 *  Notes:
 *
 *  Environment:
 *
 *========================================================================*/
{
	ssize_t	retval=0;

   UNUSED_VARIABLE(file);
   UNUSED_VARIABLE(buf);
   UNUSED_VARIABLE(nbytes);
   UNUSED_VARIABLE(ppos);

	DbgPrint( "Novfs_User_proc_read: kernel_locked 0x%x\n", kernel_locked());

	return retval;
}
/*++======================================================================*/
ssize_t
Novfs_User_proc_write(struct file * file, char * buf, size_t nbytes, loff_t *ppos)
/*
 *  Arguments:
 *
 *  Returns:
 *
 *  Abstract:
 *
 *  Notes:
 *
 *  Environment:
 *
 *========================================================================*/
{
	ssize_t	retval=0;

   UNUSED_VARIABLE(file);
   UNUSED_VARIABLE(ppos);
	
	DbgPrint( "Novfs_User_proc_write: kernel_locked 0x%x\n", kernel_locked());
	if (buf && nbytes)
	{
	}
	
	return(retval);
}

/*++======================================================================*/
int Novfs_User_proc_ioctl(struct inode *inode, struct file *file, unsigned int cmd, unsigned long arg)
/*
 *  Arguments:
 *
 *  Returns:
 *
 *  Abstract:
 *
 *  Notes:
 *
 *  Environment:
 *
 *========================================================================*/
{
	int retval=-ENOSYS;

   UNUSED_VARIABLE(inode);
   UNUSED_VARIABLE(file);
   UNUSED_VARIABLE(cmd);
   UNUSED_VARIABLE(arg);

	DbgPrint( "Novfs_User_proc_ioctl: kernel_locked 0x%x\n", kernel_locked());

	return(retval);
}

/*++======================================================================*/
int Init_Procfs_Interface( void )
/*
 *  Arguments:
 *
 *  Returns:
 *
 *  Abstract:
 *
 *  Notes:
 *
 *  Environment:
 *
 *========================================================================*/
{
	int retCode=0;
	
	Novfs_Procfs_dir = proc_mkdir(MODULE_NAME, NULL);
	if ( Novfs_Procfs_dir ) 
	{
		Novfs_Procfs_dir->owner = THIS_MODULE;
		
		Novfs_Control = create_proc_entry("Control", 0600, Novfs_Procfs_dir);

		if ( Novfs_Control )
		{
			Novfs_Control->owner = THIS_MODULE;
			Novfs_Control->size  = 0;
			memcpy(&Daemon_proc_fops, Novfs_Control->proc_fops, sizeof(struct file_operations));
			
			/*
			 * Setup our functions
          */
			Daemon_proc_fops.owner   = THIS_MODULE;
			Daemon_proc_fops.open    = Daemon_Open_Control;
			Daemon_proc_fops.release = Daemon_Close_Control;
			Daemon_proc_fops.read    = Daemon_Send_Command;
			Daemon_proc_fops.write   = Daemon_Receive_Reply;
         Daemon_proc_fops.ioctl   = Daemon_ioctl;
			
			Novfs_Control->proc_fops = &Daemon_proc_fops;
		}
		else
		{
			remove_proc_entry(MODULE_NAME, NULL);
			return(-ENOENT);
		}
      
		Novfs_Library = create_proc_entry("Library", 0666, Novfs_Procfs_dir);
      if ( Novfs_Library )
      {
			Novfs_Library->owner = THIS_MODULE;
			Novfs_Library->size  = 0;
			memcpy(&Library_proc_fops, Novfs_Library->proc_fops, sizeof(struct file_operations));
			
			/*
			 * Setup our functions
          */
			Library_proc_fops.owner   = THIS_MODULE;
         Library_proc_fops.open    = Daemon_Library_open;
			Library_proc_fops.release = Daemon_Library_close;
			Library_proc_fops.read    = Daemon_Library_read;
			Library_proc_fops.write   = Daemon_Library_write;
         Library_proc_fops.ioctl   = Daemon_Library_ioctl;
			Novfs_Library->proc_fops  = &Library_proc_fops;
      }
      else
      {
         remove_proc_entry("Control", Novfs_Procfs_dir);
         remove_proc_entry(MODULE_NAME, NULL);
			return(-ENOENT);
      }

      Novfs_Version = create_proc_read_entry("Version", 0444, Novfs_Procfs_dir, Novfs_Get_Version, NULL);
      if ( Novfs_Version )
      {
			Novfs_Version->owner = THIS_MODULE;
			Novfs_Version->size  = 0;
      }
      else
      {
         remove_proc_entry("Library", Novfs_Procfs_dir);
         remove_proc_entry("Control", Novfs_Procfs_dir);
         remove_proc_entry(MODULE_NAME, NULL);
			retCode = -ENOENT;
      }
	}
	else
	{
		retCode = -ENOENT;
	}
	return(retCode);
}

/*++======================================================================*/
void Uninit_Procfs_Interface( void )
/*
 *  Arguments:
 *
 *  Returns:
 *
 *  Abstract:
 *
 *  Notes:
 *
 *  Environment:
 *
 *========================================================================*/
{
   remove_proc_entry("Control", Novfs_Procfs_dir);
   remove_proc_entry("Library", Novfs_Procfs_dir);
   remove_proc_entry(MODULE_NAME, NULL);
}
